@@ -9,7 +9,7 @@ from . import parse, sorting, wrap
 from .comments import add_to_line as with_comments
 from .settings import DEFAULT_CONFIG, Config
 
-STATEMENT_DECLERATIONS: Tuple[str, ...] = ("def ", "cdef ", "cpdef ", "class ", "@", "async def")
+STATEMENT_DECLARATIONS: Tuple[str, ...] = ("def ", "cdef ", "cpdef ", "class ", "@", "async def")
 
 
 def sorted_imports(
@@ -29,7 +29,6 @@ def sorted_imports(
     formatted_output: List[str] = parsed.lines_without_imports.copy()
     remove_imports = [format_simplified(removal) for removal in config.remove_imports]
 
-    sort_ignore_case = config.force_alphabetical_sort_within_sections
     sections: Iterable[str] = itertools.chain(parsed.sections, config.forced_separate)
 
     if config.no_sections:
@@ -50,62 +49,34 @@ def sorted_imports(
     pending_lines_before = False
     for section in sections:
         straight_modules = parsed.imports[section]["straight"]
-        straight_modules = sorting.naturally(
-            straight_modules,
-            key=lambda key: sorting.module_key(
-                key, config, section_name=section, straight_import=True
-            ),
-        )
+        if not config.only_sections:
+            straight_modules = sorting.naturally(
+                straight_modules,
+                key=lambda key: sorting.module_key(
+                    key, config, section_name=section, straight_import=True
+                ),
+            )
+
         from_modules = parsed.imports[section]["from"]
-        from_modules = sorting.naturally(
-            from_modules, key=lambda key: sorting.module_key(key, config, section_name=section)
+        if not config.only_sections:
+            from_modules = sorting.naturally(
+                from_modules, key=lambda key: sorting.module_key(key, config, section_name=section)
+            )
+
+        straight_imports = _with_straight_imports(
+            parsed, config, straight_modules, section, remove_imports, import_type
+        )
+        from_imports = _with_from_imports(
+            parsed, config, from_modules, section, remove_imports, import_type
         )
 
-        section_output: List[str] = []
+        lines_between = [""] * (
+            config.lines_between_types if from_modules and straight_modules else 0
+        )
         if config.from_first:
-            section_output = _with_from_imports(
-                parsed,
-                config,
-                from_modules,
-                section,
-                section_output,
-                sort_ignore_case,
-                remove_imports,
-                import_type,
-            )
-            if config.lines_between_types and from_modules and straight_modules:
-                section_output.extend([""] * config.lines_between_types)
-            section_output = _with_straight_imports(
-                parsed,
-                config,
-                straight_modules,
-                section,
-                section_output,
-                remove_imports,
-                import_type,
-            )
+            section_output = from_imports + lines_between + straight_imports
         else:
-            section_output = _with_straight_imports(
-                parsed,
-                config,
-                straight_modules,
-                section,
-                section_output,
-                remove_imports,
-                import_type,
-            )
-            if config.lines_between_types and from_modules and straight_modules:
-                section_output.extend([""] * config.lines_between_types)
-            section_output = _with_from_imports(
-                parsed,
-                config,
-                from_modules,
-                section,
-                section_output,
-                sort_ignore_case,
-                remove_imports,
-                import_type,
-            )
+            section_output = straight_imports + lines_between + from_imports
 
         if config.force_sort_within_sections:
             # collapse comments
@@ -121,7 +92,7 @@ def sorted_imports(
                     comments_above = []
                 else:
                     new_section_output.append(line)
-
+            # only_sections options is not imposed if force_sort_within_sections is True
             new_section_output = sorting.naturally(
                 new_section_output,
                 key=partial(
@@ -130,6 +101,8 @@ def sorted_imports(
                     force_to_top=config.force_to_top,
                     lexicographical=config.lexicographical,
                     length_sort=config.length_sort,
+                    reverse_relative=config.reverse_relative,
+                    group_by_package=config.group_by_package,
                 ),
             )
 
@@ -138,12 +111,6 @@ def sorted_imports(
             for line in new_section_output:
                 comments = getattr(line, "comments", ())
                 if comments:
-                    if (
-                        config.ensure_newline_before_comments
-                        and section_output
-                        and section_output[-1]
-                    ):
-                        section_output.append("")
                     section_output.extend(comments)
                 section_output.append(str(line))
 
@@ -172,6 +139,9 @@ def sorted_imports(
         else:
             pending_lines_before = pending_lines_before or not no_lines_before
 
+    if config.ensure_newline_before_comments:
+        output = _ensure_newline_before_comment(output)
+
     while output and output[-1].strip() == "":
         output.pop()  # pragma: no cover
     while output and output[0].strip() == "":
@@ -220,7 +190,7 @@ def sorted_imports(
 
         if config.lines_after_imports != -1:
             formatted_output[imports_tail:0] = ["" for line in range(config.lines_after_imports)]
-        elif extension != "pyi" and next_construct.startswith(STATEMENT_DECLERATIONS):
+        elif extension != "pyi" and next_construct.startswith(STATEMENT_DECLARATIONS):
             formatted_output[imports_tail:0] = ["", ""]
         else:
             formatted_output[imports_tail:0] = [""]
@@ -246,12 +216,10 @@ def _with_from_imports(
     config: Config,
     from_modules: Iterable[str],
     section: str,
-    section_output: List[str],
-    ignore_case: bool,
     remove_imports: List[str],
     import_type: str,
 ) -> List[str]:
-    new_section_output = section_output.copy()
+    output: List[str] = []
     for module in from_modules:
         if module in remove_imports:
             continue
@@ -261,12 +229,19 @@ def _with_from_imports(
         if not config.no_inline_sort or (
             config.force_single_line and module not in config.single_line_exclusions
         ):
-            from_imports = sorting.naturally(
-                from_imports,
-                key=lambda key: sorting.module_key(
-                    key, config, True, ignore_case, section_name=section
-                ),
-            )
+            ignore_case = config.force_alphabetical_sort_within_sections
+
+            if not config.only_sections:
+                from_imports = sorting.naturally(
+                    from_imports,
+                    key=lambda key: sorting.module_key(
+                        key,
+                        config,
+                        True,
+                        ignore_case,
+                        section_name=section,
+                    ),
+                )
         if remove_imports:
             from_imports = [
                 line for line in from_imports if f"{module}.{line}" not in remove_imports
@@ -283,7 +258,8 @@ def _with_from_imports(
         if config.combine_as_imports and not ("*" in from_imports and config.combine_star):
             if not config.no_inline_sort:
                 for as_import in as_imports:
-                    as_imports[as_import] = sorting.naturally(as_imports[as_import])
+                    if not config.only_sections:
+                        as_imports[as_import] = sorting.naturally(as_imports[as_import])
             for from_import in copy.copy(from_imports):
                 if from_import in as_imports:
                     idx = from_imports.index(from_import)
@@ -292,23 +268,18 @@ def _with_from_imports(
                     else:
                         from_imports[idx : (idx + 1)] = as_imports.pop(from_import)
 
+        only_show_as_imports = False
+        comments = parsed.categorized_comments["from"].pop(module, ())
+        above_comments = parsed.categorized_comments["above"]["from"].pop(module, None)
         while from_imports:
-            comments = parsed.categorized_comments["from"].pop(module, ())
-            above_comments = parsed.categorized_comments["above"]["from"].pop(module, None)
             if above_comments:
-                if new_section_output and config.ensure_newline_before_comments:
-                    new_section_output.append("")
-                new_section_output.extend(above_comments)
+                output.extend(above_comments)
+                above_comments = None
 
             if "*" in from_imports and config.combine_star:
-                if config.combine_as_imports:
-                    comments = list(comments or ())
-                    comments += parsed.categorized_comments["from"].pop(
-                        f"{module}.__combined_as__", []
-                    )
                 import_statement = wrap.line(
                     with_comments(
-                        comments,
+                        _with_star_comments(parsed, module, list(comments or ())),
                         f"{import_start}*",
                         removed=config.ignore_comments,
                         comment_prefix=config.comment_prefix,
@@ -316,7 +287,10 @@ def _with_from_imports(
                     parsed.line_separator,
                     config,
                 )
-                from_imports = []
+                from_imports = [
+                    from_import for from_import in from_imports if from_import in as_imports
+                ]
+                only_show_as_imports = True
             elif config.force_single_line and module not in config.single_line_exclusions:
                 import_statement = ""
                 while from_imports:
@@ -335,36 +309,66 @@ def _with_from_imports(
                             f"{comments and ';' or config.comment_prefix} " f"{comment}"
                         )
                     if from_import in as_imports:
-                        if parsed.imports[section]["from"][module][from_import]:
-                            new_section_output.append(
+                        if (
+                            parsed.imports[section]["from"][module][from_import]
+                            and not only_show_as_imports
+                        ):
+                            output.append(
                                 wrap.line(single_import_line, parsed.line_separator, config)
                             )
                         from_comments = parsed.categorized_comments["straight"].get(
                             f"{module}.{from_import}"
                         )
-                        new_section_output.extend(
-                            with_comments(
-                                from_comments,
-                                wrap.line(import_start + as_import, parsed.line_separator, config),
-                                removed=config.ignore_comments,
-                                comment_prefix=config.comment_prefix,
+
+                        if not config.only_sections:
+                            output.extend(
+                                with_comments(
+                                    from_comments,
+                                    wrap.line(
+                                        import_start + as_import, parsed.line_separator, config
+                                    ),
+                                    removed=config.ignore_comments,
+                                    comment_prefix=config.comment_prefix,
+                                )
+                                for as_import in sorting.naturally(as_imports[from_import])
+                            )
+
+                        else:
+                            output.extend(
+                                with_comments(
+                                    from_comments,
+                                    wrap.line(
+                                        import_start + as_import, parsed.line_separator, config
+                                    ),
+                                    removed=config.ignore_comments,
+                                    comment_prefix=config.comment_prefix,
+                                )
+                                for as_import in as_imports[from_import]
                             )
-                            for as_import in sorting.naturally(as_imports[from_import])
-                        )
                     else:
-                        new_section_output.append(
-                            wrap.line(single_import_line, parsed.line_separator, config)
-                        )
+                        output.append(wrap.line(single_import_line, parsed.line_separator, config))
                     comments = None
             else:
                 while from_imports and from_imports[0] in as_imports:
                     from_import = from_imports.pop(0)
-                    as_imports[from_import] = sorting.naturally(as_imports[from_import])
-                    from_comments = parsed.categorized_comments["straight"].get(
-                        f"{module}.{from_import}"
+
+                    if not config.only_sections:
+                        as_imports[from_import] = sorting.naturally(as_imports[from_import])
+                    from_comments = (
+                        parsed.categorized_comments["straight"].get(f"{module}.{from_import}") or []
                     )
-                    if parsed.imports[section]["from"][module][from_import]:
-                        new_section_output.append(
+                    if (
+                        parsed.imports[section]["from"][module][from_import]
+                        and not only_show_as_imports
+                    ):
+                        specific_comment = (
+                            parsed.categorized_comments["nested"]
+                            .get(module, {})
+                            .pop(from_import, None)
+                        )
+                        if specific_comment:
+                            from_comments.append(specific_comment)
+                        output.append(
                             wrap.line(
                                 with_comments(
                                     from_comments,
@@ -376,24 +380,36 @@ def _with_from_imports(
                                 config,
                             )
                         )
-                    new_section_output.extend(
-                        wrap.line(
-                            with_comments(
-                                from_comments,
-                                import_start + as_import,
-                                removed=config.ignore_comments,
-                                comment_prefix=config.comment_prefix,
-                            ),
-                            parsed.line_separator,
-                            config,
+                        from_comments = []
+
+                    for as_import in as_imports[from_import]:
+                        specific_comment = (
+                            parsed.categorized_comments["nested"]
+                            .get(module, {})
+                            .pop(as_import, None)
                         )
-                        for as_import in as_imports[from_import]
-                    )
+                        if specific_comment:
+                            from_comments.append(specific_comment)
+
+                        output.append(
+                            wrap.line(
+                                with_comments(
+                                    from_comments,
+                                    import_start + as_import,
+                                    removed=config.ignore_comments,
+                                    comment_prefix=config.comment_prefix,
+                                ),
+                                parsed.line_separator,
+                                config,
+                            )
+                        )
+
+                        from_comments = []
 
                 if "*" in from_imports:
-                    new_section_output.append(
+                    output.append(
                         with_comments(
-                            comments,
+                            _with_star_comments(parsed, module, list(comments or ())),
                             f"{import_start}*",
                             removed=config.ignore_comments,
                             comment_prefix=config.comment_prefix,
@@ -416,9 +432,7 @@ def _with_from_imports(
                         single_import_line += (
                             f"{comments and ';' or config.comment_prefix} " f"{comment}"
                         )
-                        new_section_output.append(
-                            wrap.line(single_import_line, parsed.line_separator, config)
-                        )
+                        output.append(wrap.line(single_import_line, parsed.line_separator, config))
                         from_imports.remove(from_import)
                         comments = None
 
@@ -486,8 +500,8 @@ def _with_from_imports(
                     import_statement = wrap.line(import_statement, parsed.line_separator, config)
 
             if import_statement:
-                new_section_output.append(import_statement)
-    return new_section_output
+                output.append(import_statement)
+    return output
 
 
 def _with_straight_imports(
@@ -495,11 +509,10 @@ def _with_straight_imports(
     config: Config,
     straight_modules: Iterable[str],
     section: str,
-    section_output: List[str],
     remove_imports: List[str],
     import_type: str,
 ) -> List[str]:
-    new_section_output = section_output.copy()
+    output: List[str] = []
     for module in straight_modules:
         if module in remove_imports:
             continue
@@ -517,10 +530,8 @@ def _with_straight_imports(
 
         comments_above = parsed.categorized_comments["above"]["straight"].pop(module, None)
         if comments_above:
-            if new_section_output and config.ensure_newline_before_comments:
-                new_section_output.append("")
-            new_section_output.extend(comments_above)
-        new_section_output.extend(
+            output.extend(comments_above)
+        output.extend(
             with_comments(
                 parsed.categorized_comments["straight"].get(module),
                 idef,
@@ -530,7 +541,7 @@ def _with_straight_imports(
             for idef in import_definition
         )
 
-    return new_section_output
+    return output
 
 
 def _output_as_string(lines: List[str], line_separator: str) -> str:
@@ -550,3 +561,24 @@ class _LineWithComments(str):
         instance = super().__new__(cls, value)  # type: ignore
         instance.comments = comments
         return instance
+
+
+def _ensure_newline_before_comment(output):
+    new_output: List[str] = []
+
+    def is_comment(line):
+        return line and line.startswith("#")
+
+    for line, prev_line in zip(output, [None] + output):
+        if is_comment(line) and prev_line != "" and not is_comment(prev_line):
+            new_output.append("")
+        new_output.append(line)
+    return new_output
+
+
+def _with_star_comments(parsed: parse.ParsedContent, module: str, comments: List[str]) -> List[str]:
+    star_comment = parsed.categorized_comments["nested"].get(module, {}).pop("*", None)
+    if star_comment:
+        return comments + [star_comment]
+    else:
+        return comments
