@@ -113,7 +113,7 @@ def skip_line(
                 break
             char_index += 1
 
-    if ";" in line and needs_import:
+    if ";" in line.split("#")[0] and needs_import:
         for part in (part.strip() for part in line.split(";")):
             if (
                 part
@@ -138,6 +138,7 @@ class ParsedContent(NamedTuple):
     original_line_count: int
     line_separator: str
     sections: Any
+    verbose_output: List[str]
 
 
 def file_contents(contents: str, config: Config = DEFAULT_CONFIG) -> ParsedContent:
@@ -163,6 +164,8 @@ def file_contents(contents: str, config: Config = DEFAULT_CONFIG) -> ParsedConte
         "from": defaultdict(list),
     }
     imports: OrderedDict[str, Dict[str, Any]] = OrderedDict()
+    verbose_output: List[str] = []
+
     for section in chain(config.sections, config.forced_separate):
         imports[section] = {"straight": OrderedDict(), "from": OrderedDict()}
     categorized_comments: CommentsDict = {
@@ -201,10 +204,61 @@ def file_contents(contents: str, config: Config = DEFAULT_CONFIG) -> ParsedConte
             out_lines.append(line)
             continue
 
-        for line in (
-            (line.strip() for line in line.split(";")) if ";" in line else (line,)  # type: ignore
+        lstripped_line = line.lstrip()
+        if (
+            config.float_to_top
+            and import_index == -1
+            and line
+            and not in_quote
+            and not lstripped_line.startswith("#")
+            and not lstripped_line.startswith("'''")
+            and not lstripped_line.startswith('"""')
         ):
-            line, raw_line = _normalize_line(line)
+            if not lstripped_line.startswith("import") and not lstripped_line.startswith("from"):
+                import_index = index - 1
+                while import_index and not in_lines[import_index - 1]:
+                    import_index -= 1
+            else:
+                commentless = line.split("#", 1)[0].strip()
+                if (
+                    ("isort:skip" in line or "isort: skip" in line)
+                    and "(" in commentless
+                    and ")" not in commentless
+                ):
+                    import_index = index
+
+                    starting_line = line
+                    while "isort:skip" in starting_line or "isort: skip" in starting_line:
+                        commentless = starting_line.split("#", 1)[0]
+                        if (
+                            "(" in commentless
+                            and not commentless.rstrip().endswith(")")
+                            and import_index < line_count
+                        ):
+
+                            while import_index < line_count and not commentless.rstrip().endswith(
+                                ")"
+                            ):
+                                commentless = in_lines[import_index].split("#", 1)[0]
+                                import_index += 1
+                        else:
+                            import_index += 1
+
+                        if import_index >= line_count:
+                            break
+                        else:
+                            starting_line = in_lines[import_index]
+
+        line, *end_of_line_comment = line.split("#", 1)
+        if ";" in line:
+            statements = [line.strip() for line in line.split(";")]
+        else:
+            statements = [line]
+        if end_of_line_comment:
+            statements[-1] = f"{statements[-1]}#{end_of_line_comment[0]}"
+
+        for statement in statements:
+            line, raw_line = _normalize_line(statement)
             type_of_import = import_type(line, config) or ""
             if not type_of_import:
                 out_lines.append(raw_line)
@@ -216,15 +270,10 @@ def file_contents(contents: str, config: Config = DEFAULT_CONFIG) -> ParsedConte
             import_string, comment = parse_comments(line)
             comments = [comment] if comment else []
             line_parts = [part for part in _strip_syntax(import_string).strip().split(" ") if part]
-            if (
-                type_of_import == "from"
-                and len(line_parts) == 2
-                and line_parts[1] != "*"
-                and comments
-            ):
+            if type_of_import == "from" and len(line_parts) == 2 and comments:
                 nested_comments[line_parts[-1]] = comments[0]
 
-            if "(" in line.split("#")[0] and index < line_count:
+            if "(" in line.split("#", 1)[0] and index < line_count:
                 while not line.split("#")[0].strip().endswith(")") and index < line_count:
                     line, new_comment = parse_comments(in_lines[index])
                     index += 1
@@ -234,7 +283,7 @@ def file_contents(contents: str, config: Config = DEFAULT_CONFIG) -> ParsedConte
                     if (
                         type_of_import == "from"
                         and stripped_line
-                        and " " not in stripped_line
+                        and " " not in stripped_line.replace(" as ", "")
                         and new_comment
                     ):
                         nested_comments[stripped_line] = comments[-1]
@@ -256,7 +305,7 @@ def file_contents(contents: str, config: Config = DEFAULT_CONFIG) -> ParsedConte
                         if (
                             type_of_import == "from"
                             and stripped_line
-                            and " " not in stripped_line
+                            and " " not in stripped_line.replace(" as ", "")
                             and new_comment
                         ):
                             nested_comments[stripped_line] = comments[-1]
@@ -271,7 +320,7 @@ def file_contents(contents: str, config: Config = DEFAULT_CONFIG) -> ParsedConte
                             if (
                                 type_of_import == "from"
                                 and stripped_line
-                                and " " not in stripped_line
+                                and " " not in stripped_line.replace(" as ", "")
                                 and new_comment
                             ):
                                 nested_comments[stripped_line] = comments[-1]
@@ -281,7 +330,7 @@ def file_contents(contents: str, config: Config = DEFAULT_CONFIG) -> ParsedConte
                     if (
                         type_of_import == "from"
                         and stripped_line
-                        and " " not in stripped_line
+                        and " " not in stripped_line.replace(" as ", "")
                         and new_comment
                     ):
                         nested_comments[stripped_line] = comments[-1]
@@ -316,6 +365,9 @@ def file_contents(contents: str, config: Config = DEFAULT_CONFIG) -> ParsedConte
                 item.replace("{|", "{ ").replace("|}", " }")
                 for item in _strip_syntax(import_string).split()
             ]
+
+            attach_comments_to: Optional[List[Any]] = None
+            direct_imports = just_imports[1:]
             straight_import = True
             if "as" in just_imports and (just_imports.index("as") + 1) < len(just_imports):
                 straight_import = False
@@ -327,10 +379,22 @@ def file_contents(contents: str, config: Config = DEFAULT_CONFIG) -> ParsedConte
                         top_level_module = just_imports[0]
                         module = top_level_module + "." + nested_module
                         as_name = just_imports[as_index + 1]
+                        direct_imports.remove(nested_module)
+                        direct_imports.remove(as_name)
+                        direct_imports.remove("as")
                         if nested_module == as_name and config.remove_redundant_aliases:
                             pass
                         elif as_name not in as_map["from"][module]:
                             as_map["from"][module].append(as_name)
+
+                        full_name = f"{nested_module} as {as_name}"
+                        associated_comment = nested_comments.get(full_name)
+                        if associated_comment:
+                            categorized_comments["nested"].setdefault(top_level_module, {})[
+                                full_name
+                            ] = associated_comment
+                            if associated_comment in comments:
+                                comments.pop(comments.index(associated_comment))
                     else:
                         module = just_imports[as_index - 1]
                         as_name = just_imports[as_index + 1]
@@ -339,20 +403,27 @@ def file_contents(contents: str, config: Config = DEFAULT_CONFIG) -> ParsedConte
                         elif as_name not in as_map["straight"][module]:
                             as_map["straight"][module].append(as_name)
 
-                    if config.combine_as_imports and nested_module:
-                        categorized_comments["from"].setdefault(
-                            f"{top_level_module}.__combined_as__", []
-                        ).extend(comments)
-                        comments = []
-                    else:
-                        categorized_comments["straight"][module] = comments
-                        comments = []
+                    if comments and attach_comments_to is None:
+                        if nested_module and config.combine_as_imports:
+                            attach_comments_to = categorized_comments["from"].setdefault(
+                                f"{top_level_module}.__combined_as__", []
+                            )
+                        else:
+                            attach_comments_to = categorized_comments["straight"].setdefault(
+                                module, []
+                            )
                     del just_imports[as_index : as_index + 2]
+
             if type_of_import == "from":
                 import_from = just_imports.pop(0)
                 placed_module = finder(import_from)
-                if config.verbose:
+                if config.verbose and not config.only_modified:
                     print(f"from-type place_module for {import_from} returned {placed_module}")
+
+                elif config.verbose:
+                    verbose_output.append(
+                        f"from-type place_module for {import_from} returned {placed_module}"
+                    )
                 if placed_module == "":
                     warn(
                         f"could not place module {import_from} of line {line} --"
@@ -367,8 +438,9 @@ def file_contents(contents: str, config: Config = DEFAULT_CONFIG) -> ParsedConte
                         ] = associated_comment
                         if associated_comment in comments:
                             comments.pop(comments.index(associated_comment))
-                if comments:
-                    categorized_comments["from"].setdefault(import_from, []).extend(comments)
+
+                if comments and attach_comments_to is None:
+                    attach_comments_to = categorized_comments["from"].setdefault(import_from, [])
 
                 if len(out_lines) > max(import_index, 1) - 1:
                     last = out_lines and out_lines[-1].rstrip() or ""
@@ -395,14 +467,21 @@ def file_contents(contents: str, config: Config = DEFAULT_CONFIG) -> ParsedConte
 
                 if import_from not in root:
                     root[import_from] = OrderedDict(
-                        (module, straight_import) for module in just_imports
+                        (module, module in direct_imports) for module in just_imports
                     )
                 else:
                     root[import_from].update(
-                        (module, straight_import | root[import_from].get(module, False))
+                        (module, root[import_from].get(module, False) or module in direct_imports)
                         for module in just_imports
                     )
+
+                if comments and attach_comments_to is not None:
+                    attach_comments_to.extend(comments)
             else:
+                if comments and attach_comments_to is not None:
+                    attach_comments_to.extend(comments)
+                    comments = []
+
                 for module in just_imports:
                     if comments:
                         categorized_comments["straight"][module] = comments
@@ -432,8 +511,13 @@ def file_contents(contents: str, config: Config = DEFAULT_CONFIG) -> ParsedConte
                                 categorized_comments["above"]["straight"].get(module, [])
                             )
                     placed_module = finder(module)
-                    if config.verbose:
+                    if config.verbose and not config.only_modified:
                         print(f"else-type place_module for {module} returned {placed_module}")
+
+                    elif config.verbose:
+                        verbose_output.append(
+                            f"else-type place_module for {module} returned {placed_module}"
+                        )
                     if placed_module == "":
                         warn(
                             f"could not place module {module} of line {line} --"
@@ -460,4 +544,5 @@ def file_contents(contents: str, config: Config = DEFAULT_CONFIG) -> ParsedConte
         original_line_count=original_line_count,
         line_separator=line_separator,
         sections=config.sections,
+        verbose_output=verbose_output,
     )
